@@ -63,7 +63,7 @@ IAuth <- R6Class(
 #'   \item{`password`}{the user password}
 #' }
 #'
-#' @return an object of type [R6Class()] representing basic authentication
+#' @return an object of type [R6::R6Class()] representing basic authentication
 #' @importFrom R6 R6Class
 NULL
 
@@ -168,6 +168,8 @@ BasicAuth <- R6Class(
 #' \itemize{
 #'   \item{authorization_code}
 #'   \item{authorization_code+pkce}
+#'   \item{client_credentials}
+#'   \item{urn:ietf:params:oauth:grant-type:device_code}
 #'   \item{urn:ietf:params:oauth:grant-type:device_code+pkce}
 #' }
 #' 
@@ -225,7 +227,9 @@ AbstractOIDCAuthentication <- R6Class(
       args = list(...)
       if ("force" %in% names(args)) private$force_use = args[["force"]]
       # comfort function select provider by name if one is provided
-      provider = .get_oidc_provider(provider)
+      if (!inherits(provider, "Provider")) {
+        provider = .get_oidc_provider(provider)
+      }
       
       private$setIssuer(provider$issuer)
       
@@ -246,15 +250,19 @@ AbstractOIDCAuthentication <- R6Class(
       }
       
       # user knows best, allow custom scopes...
-      if (length(config$scopes) > 0 && is.character(config$scopes)) {
-        private$scopes = config$scopes
+      if (length(config$scopes) > 0) {
+        if (is.character(config$scopes)) {
+          private$scopes = list(config$scopes)
+        } else {
+          private$scopes = config$scopes
+        }
       } else if (length(provider$scopes) == 0) {
         private$scopes = list("openid")
       } else {
         private$scopes = provider$scopes
-        
+
         #TODO remove later, this is used for automatic reconnect
-        if (!"offline_access" %in% private$scopes) {
+        if (private$grant_type != "client_credentials" && !"offline_access" %in% private$scopes) {
           private$scopes = c(private$scopes, "offline_access")
         }
       }
@@ -267,8 +275,8 @@ AbstractOIDCAuthentication <- R6Class(
       
       private$client_id = config$client_id
       
-      if (private$grant_type == "authorization_code") {
-        # in this case we need a client_id and secrect, which is basically the old OIDC Auth Code implementation
+      if (private$grant_type == "authorization_code" || private$grant_type == "client_credentials") {
+        # in this case we need a client_id and secrect
         if (!all(c("client_id","secret") %in% names(config))) {
           stop("'client_id' and 'secret' are not present in the configuration.")
         }
@@ -280,7 +288,6 @@ AbstractOIDCAuthentication <- R6Class(
           secret = config$secret
         )
       } else {
-      
         private$oauth_client = oauth_client(
           id = private$client_id,
           token_url = private$endpoints$token_endpoint,
@@ -446,16 +453,9 @@ OIDCDeviceCodeFlow <- R6Class(
   public = list(
     # functions ####
     login = function() {
-      
-      client <- oauth_client(
-        id = private$client_id,
-        token_url = private$endpoints$token_endpoint,
-        name = "openeo-r-oidc-auth"
-      )
-
       private$auth = rlang::with_interactive(
                       oauth_flow_device(
-                        client = client,
+                        client = private$oauth_client,
                         auth_url = private$endpoints$device_authorization_endpoint,
                         scope = paste0(private$scopes, collapse = " ")
                       ),
@@ -488,16 +488,9 @@ OIDCDeviceCodeFlowPkce <- R6Class(
   public = list(
     # functions ####
     login = function() {
-      
-      client <- oauth_client(
-        id = private$client_id,
-        token_url = private$endpoints$token_endpoint,
-        name = "openeo-r-oidc-auth"
-      )
-
       private$auth = rlang::with_interactive(
                       oauth_flow_device(
-                        client = client,
+                        client = private$oauth_client,
                         auth_url = private$endpoints$device_authorization_endpoint,
                         scope = paste0(private$scopes, collapse = " "),
                         pkce = TRUE
@@ -533,16 +526,9 @@ OIDCAuthCodeFlowPKCE <- R6Class(
     # attributes ####
     # functions ####
     login = function() {
-
-      client <- oauth_client(
-        id = private$client_id,
-        token_url = private$endpoints$token_endpoint,
-        name = "openeo-r-oidc-auth"
-      )
-
       private$auth = rlang::with_interactive(
                       oauth_flow_auth_code(
-                        client = client,
+                        client = private$oauth_client,
                         auth_url = private$endpoints$authorization_endpoint,
                         scope = paste0(private$scopes, collapse = " "),
                         pkce = TRUE,
@@ -580,15 +566,9 @@ OIDCAuthCodeFlow <- R6Class(
     
     # functions ####
     login = function() {
-      client <- oauth_client(
-        id = private$client_id,
-        token_url = private$endpoints$token_endpoint,
-        name = "openeo-r-oidc-auth"
-      )
-
       private$auth = rlang::with_interactive(
                       oauth_flow_auth_code(
-                        client = client,
+                        client = private$oauth_client,
                         auth_url = private$endpoints$authorization_endpoint,
                         scope = paste0(private$scopes, collapse = " "),
                         pkce = FALSE,
@@ -616,8 +596,43 @@ OIDCAuthCodeFlow <- R6Class(
   )
 )
 
+# [OIDCClientCredentialsFlow] ----
+OIDCClientCredentialsFlow <- R6Class(
+  "OIDCClientCredentialsFlow",
+  inherit = AbstractOIDCAuthentication,
+  # public ====
+  public = list(
+    # functions ####
+    login = function() {
+      private$auth = oauth_flow_client_credentials(
+                        client = private$oauth_client,
+                        scope = paste0(private$scopes, collapse = " ")
+                      )
+
+      invisible(self)
+    }
+  ),
+  # private ====
+  private = list(
+    # attributes ####
+    grant_type = "client_credentials", # not used internally by httr2, but maybe useful in openeo
+    
+    # functions ####
+    isGrantTypeSupported = function(grant_types) {
+      if (!"client_credentials" %in% grant_types) {
+        stop("Client Credentials flow is not supported by the authentication provider")
+      }
+      invisible(TRUE)
+    }
+  )
+)
+
 # utility functions ----
 .get_oidc_provider = function(provider, oidc_providers=NULL) {
+  if (is.debugging()) {
+    if (inherits(provider,"Provider")) return(provider)
+  }
+  
   if (is.null(oidc_providers)) {
     oidc_providers = list_oidc_providers()
   }
